
#pragma once

#include "JuceHeader.h"
#include "HostAudioPlayHead.h"
#include "IPlayControl.h"

class AudioController : public IPlayControl {
   public:
    using AudioGraphIOProcessor = AudioProcessorGraph::AudioGraphIOProcessor;
    using Node                  = AudioProcessorGraph::Node;

    AudioController(double sampling_rate);
    ~AudioController();

    /* IPlayControl */
    void TransportPlay(bool should_start_playing, NotificationType notification = dontSendNotification) override;
    void MovePosition(MoveType type) override;
    void SetPlayPosition(double new_time_in_seconds) override;

   private:
    void PrepareToPlay(int samples_per_block_expected, double sample_rate);
    void ReleaseResources() { main_processor_->releaseResources(); }
    void ProcessBlock(const AudioSourceChannelInfo& buffer_to_fill);

    std::unique_ptr<juce::AudioProcessorGraph> main_processor_;
    Node::Ptr audio_output_node_;
    Node::Ptr plugin_node_;

    HostAudioPlayHead audio_play_head_;
    CriticalSection cs_;

    friend class MainComponent;
    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(AudioController)
};