
#include "AudioFileProcessor.h"

AudioFileProcessor::AudioFileProcessor(){
    setChannelLayoutOfBus(true, 0, AudioChannelSet::canonicalChannelSet(2));
    setChannelLayoutOfBus(false, 0, AudioChannelSet::canonicalChannelSet(2));

    if (auto* reader = WavAudioFormat().createReaderFor(
        new MemoryInputStream(BinaryData::sample_audio_wav, BinaryData::sample_audio_wavSize, true), true)) {
        std::unique_ptr<juce::AudioFormatReaderSource> new_source(new juce::AudioFormatReaderSource(reader, true));
        transport_source_.setSource(new_source.get(), 0, nullptr, reader->sampleRate);
        reader_source_.reset(new_source.release());
    }
}

AudioFileProcessor::~AudioFileProcessor() {}

void AudioFileProcessor::prepareToPlay(double sampleRate, int maximumExpectedSamplesPerBlock) {
    transport_source_.prepareToPlay(maximumExpectedSamplesPerBlock, sampleRate);
    transport_source_.setNextReadPosition(next_position_);
}

void AudioFileProcessor::releaseResources() { transport_source_.releaseResources(); }

void AudioFileProcessor::processBlock(AudioBuffer<float>& buffer, MidiBuffer& /*midiMessages*/) {
    if (reader_source_.get() == nullptr) {
        buffer.clear();
        return;
    }

    auto* play_head = getPlayHead();
    if (play_head == nullptr) return;

    AudioPlayHead::CurrentPositionInfo position_info;
    if (!play_head->getCurrentPosition(position_info)) return;

    if (position_info.isPlaying && !transport_source_.isPlaying())
        transport_source_.start();
    else if (!position_info.isPlaying && transport_source_.isPlaying())
        transport_source_.stop();

    // Note: Don't use getCurrentPosition() for Audio files with different sampling rates, as this will cause errors.
    if (next_position_ != position_info.timeInSamples) {
        transport_source_.setNextReadPosition(position_info.timeInSamples);
        next_position_ = position_info.timeInSamples;
    }
    transport_source_.getNextAudioBlock(AudioSourceChannelInfo(buffer));

    if (transport_source_.isPlaying()) {
        next_position_ += buffer.getNumSamples();
    }
}
