#include "MainComponent.h"
#include "ControlPanel.h"
#include "AudioController.h"

//==============================================================================
MainComponent::MainComponent()
{
    audio_controller_.reset(new AudioController(48000));

    control_panel_.reset(new ControlPanel(*audio_controller_.get()));
    addAndMakeVisible(control_panel_.get());

    setSize(800, 100);
    setAudioChannels(0, 2);
}

MainComponent::~MainComponent()
{
    shutdownAudio();
}

//==============================================================================
void MainComponent::prepareToPlay (int samplesPerBlockExpected, double sampleRate)
{
    if (audio_controller_ != nullptr) {
        audio_controller_->PrepareToPlay(samplesPerBlockExpected, sampleRate);
    }
}

void MainComponent::getNextAudioBlock (const juce::AudioSourceChannelInfo& bufferToFill)
{
    bufferToFill.clearActiveBufferRegion();
    audio_controller_->ProcessBlock(bufferToFill);
}

void MainComponent::releaseResources()
{
	if (audio_controller_ != nullptr) {
        audio_controller_->ReleaseResources();
    }
}

//==============================================================================
void MainComponent::paint (juce::Graphics& g)
{
    g.fillAll (getLookAndFeel().findColour (juce::ResizableWindow::backgroundColourId));
}

void MainComponent::resized()
{
    if (control_panel_ != nullptr) {
        control_panel_->setBounds(getLocalBounds());
    }
}
